package com.flyfish.framework.dict.config;

import com.flyfish.framework.annotations.DictValue;
import com.flyfish.framework.dict.domain.Dictionary;
import com.flyfish.framework.dict.domain.DictionaryValue;
import com.flyfish.framework.dict.service.DictionaryService;
import com.flyfish.framework.enums.BlankEnum;
import com.flyfish.framework.enums.NamedEnum;
import com.flyfish.framework.utils.Assert;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.reflections.Reflections;
import org.reflections.scanners.FieldAnnotationsScanner;
import org.reflections.util.ClasspathHelper;
import org.reflections.util.ConfigurationBuilder;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.net.URL;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 字典处理器
 *
 * @author wangyu
 * 用于反向注册默认值
 */
@RequiredArgsConstructor
@Component
public class DictionaryProcessor implements InitializingBean {

    private final DictionaryService dictionaryService;

    @Override
    public void afterPropertiesSet() {
        String[] basePackages = DictionaryConfig.basePackages();
        Assert.notNull(basePackages, "未指定明确的字典扫描路径！");
        Collection<URL> urls = Arrays.stream(basePackages).flatMap(pack -> ClasspathHelper.forPackage(pack).stream())
                .collect(Collectors.toList());
        Reflections reflections = new Reflections(new ConfigurationBuilder()
                .setUrls(urls)
                .setScanners(new FieldAnnotationsScanner()));
        Set<Field> fields = reflections.getFieldsAnnotatedWith(DictValue.class);
        if (CollectionUtils.isNotEmpty(fields)) {
            // 查找是否存在，不存在插入，存在无视
            List<Dictionary> dictionaries = fields.stream().map(field -> field.getAnnotation(DictValue.class))
                    .filter(annotation -> null != annotation && StringUtils.isNotBlank(annotation.value()) &&
                            BlankEnum.class != annotation.enumType())
                    .map(annotation -> dictionaryService
                            .getByCode(annotation.value())
                            .map(dictionary -> applyDictProps(dictionary, annotation))
                            .orElseGet(() -> applyDictProps(new Dictionary(), annotation)))
                    .distinct()
                    .collect(Collectors.toList());
            dictionaryService.updateBatch(dictionaries);
        }
    }

    /**
     * 给字典属性赋值
     *
     * @param dictionary 字典
     * @param annotation 注解
     * @return 结果
     */
    private Dictionary applyDictProps(Dictionary dictionary, DictValue annotation) {
        dictionary.setCode(annotation.value());
        if (CollectionUtils.isEmpty(dictionary.getValues())) {
            dictionary.setValues(mapValues(annotation.enumType()));
        }
        if (StringUtils.isBlank(dictionary.getDescription())) {
            dictionary.setDescription("系统自动创建的内置字典");
        }
        dictionary.setName(StringUtils.isNotBlank(annotation.name()) ? annotation.name() : annotation.value());
        return dictionary;
    }

    /**
     * 映射值，转换为字典表的值
     *
     * @return 结果
     */
    private List<DictionaryValue> mapValues(Class<? extends NamedEnum> enumClass) {
        return Arrays.stream(enumClass.getEnumConstants()).map(item -> {
            DictionaryValue value = new DictionaryValue();
            value.setEnable(true);
            value.setText(item.getName());
            value.setValue(((Enum<?>) item).name());
            return value;
        }).collect(Collectors.toList());
    }

}
